import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:rtchat/models/channels.dart';

class Viewers {
  final List<String> broadcaster;
  final List<String> moderators;
  final List<String> vips;
  final List<String> viewers;

  Viewers({
    required this.broadcaster,
    required this.moderators,
    required this.vips,
    required this.viewers,
  });

  // create a new instance that filters to viewers matching a certain substring.
  Viewers query(String text) {
    if (text.isEmpty) {
      return this;
    }
    return Viewers(
      broadcaster: broadcaster
          .where((name) => name.toLowerCase().contains(text.toLowerCase()))
          .toList(),
      moderators: moderators
          .where((name) => name.toLowerCase().contains(text.toLowerCase()))
          .toList(),
      vips: vips
          .where((name) => name.toLowerCase().contains(text.toLowerCase()))
          .toList(),
      viewers: viewers
          .where((name) => name.toLowerCase().contains(text.toLowerCase()))
          .toList(),
    );
  }

  // flatten the viewers to a list
  List<String> flatten() {
    return broadcaster + moderators + vips + viewers;
  }
}

class ChatStateModel extends ChangeNotifier {
  StreamSubscription<void>? _subscription;
  Viewers? _viewers;
  Channel? _channel;

  set channel(Channel? channel) {
    // ignore if no update
    if (channel == _channel) {
      return;
    }
    _channel = channel;
    _viewers = null;
    notifyListeners();

    _subscription?.cancel();
    if (channel != null) {
      _subscription = FirebaseFirestore.instance
          .collection('chat-status')
          .where("channelId", isEqualTo: channel.toString())
          .orderBy("createdAt", descending: true)
          .limit(1)
          .snapshots()
          .listen((snapshot) {
        if (snapshot.docs.isEmpty) {
          return;
        }
        final doc = snapshot.docs.first.data();
        _viewers = Viewers(
          broadcaster: <String>[...(doc['broadcaster'] ?? [])],
          moderators: <String>[...(doc['moderators'] ?? [])],
          vips: <String>[...(doc['vips'] ?? [])],
          viewers: <String>[...(doc['viewers'] ?? [])],
        );
        notifyListeners();
      });
    }
  }

  // can be null if still loading.
  Viewers? get viewers => _viewers;
}
